/*
 * spi.c
 *
 *     Created on: Sep 30, 2014
 *      Author: B52320
 */

#include "spi.h"
/*
 * SPI0_mstrInit initializes the SPI0 as a master 
 */

void SPI0_mstrInit()
{

	SIM_SCGC6 |= SIM_SCGC6_SPI0_MASK;	//SPI0 Clock Gate enabled

	SIM_SCGC5 |= SIM_SCGC5_PORTA_MASK;	//Port A Clock enabled

	PORTA_PCR17 = PORT_PCR_MUX(0x02); 	//SPI0_SIN //B59

	PORTA_PCR16 =  PORT_PCR_MUX(0x02); 	//SPI0_SOUT //B60

	PORTA_PCR15 =  PORT_PCR_MUX(0x02); 	//SPI0_SCK //A35

	PORTA_PCR14 = PORT_PCR_MUX(0x02); 	//SPI0_PCS0  //B35

	SPI0_MCR = SPI_MCR_MSTR_MASK |  	//DSPI is in Master mode.
				SPI_MCR_ROOE_MASK |  	//Incoming data is shifted into the shift register.
				SPI_MCR_PCSIS(0x01) |	//The inactive state of PCSx is high.
				SPI_MCR_CLR_TXF_MASK |	//Clear the TX FIFO counter.
				SPI_MCR_CLR_RXF_MASK |  //Clear the RX FIFO counter.
				SPI_MCR_HALT_MASK;		//Stop transfers(DISABLE)		

	//SPI0_MCR |= SPI_MCR_DIS_TXF_MASK |	//Tx FIFO is disabled.
	//			SPI_MCR_DIS_RXF_MASK;    //Rx FIFO is disabled.	;
	
	
	SPI0_SR |= SPI_SR_TCF_MASK |	//Clear transfer complete flag
			   SPI_SR_EOQF_MASK |	//Clear End of Queue Flag
			   SPI_SR_TFUF_MASK |	//CLear Transmit FIFO Underflow Flag
			   SPI_SR_TFFF_MASK ;	//Transmit FIFO Fill Flag
	
			   
	SPI0_CTAR0 = SPI_CTAR_FMSZ(0x07); //Frame Size = 7 + 1		   
	
	SPI0_RSER |= SPI_RSER_RFDF_RE_MASK | SPI_RSER_RFDF_DIRS_MASK |  SPI_RSER_TFFF_DIRS_MASK;   
	/* Set DMA Interrupt Request Select and Enable register */

	SPI0_MCR &= ~SPI_MCR_HALT_MASK;	//Start transfers.(ENABLE)
		
	SPI0_RSER |= SPI_RSER_TFFF_RE_MASK; //Enable the SPI0 DMA requests
}

/*
 * Master send data, this must be used when the DMA channels are active
 */

void Master_sendDMA () 
{		
	//DMA_SERQ=0x00;//while (1) {}
	
	while(!(SPI0_SR & SPI_SR_EOQF_MASK)) {} //wait till the last entry has been transmitted

	SPI0_SR  |= SPI_SR_EOQF_MASK; //clear EOQF flag
	//DMA_SERQ = 0X01;
}


/*
 * This function is used to fill the Buffer with different data to make some tests, 
 * sending different data every loop
 */

void MasterDataSend (uint8_t sequence)
{
	MstrDataSend_Buff [0] =  0x01 ;
	MstrDataSend_Buff [1] =  0x02 ;
	MstrDataSend_Buff [2] =  0x03 ;
	MstrDataSend_Buff [3] =  0x04 ;
	MstrDataSend_Buff [4] =  0x05 ;
	MstrDataSend_Buff [5] =  0x06 ;
	MstrDataSend_Buff [6] =  0x07 ;
	MstrDataSend_Buff [7] =  0x08 ;
	
	int i = 0;
	switch (sequence){
	case (0):			
			for(i = 0 ; i <= 6 ; i++){
				MstrDataSend_Buff [i] = (MstrDataSend_Buff [i] + 0xA0)| SPI_PUSHR_CTCNT_MASK | SPI_PUSHR_PCS(0x01) ;   
			}
	            MstrDataSend_Buff [7] =  (0x08 + 0xA0) | SPI_PUSHR_CTCNT_MASK | SPI_PUSHR_PCS(0x01) | SPI_PUSHR_EOQ_MASK;
			break;
	case(1):
			for(i = 0 ; i <= 6 ; i++){
				MstrDataSend_Buff [i] = (0xB0 + MstrDataSend_Buff [i])| SPI_PUSHR_CTCNT_MASK | SPI_PUSHR_PCS(0x01);  ; 
			}
				MstrDataSend_Buff [7] =  (0x08 + 0xB0) | SPI_PUSHR_CTCNT_MASK | SPI_PUSHR_PCS(0x01) | SPI_PUSHR_EOQ_MASK;
			break;
	case(2):
			for(i = 0 ; i <= 6 ; i++){
				MstrDataSend_Buff [i] = (0xC0 + MstrDataSend_Buff [i])| SPI_PUSHR_CTCNT_MASK | SPI_PUSHR_PCS(0x01);  ; 
			}
				MstrDataSend_Buff [7] =  (0x08 + 0xC0) | SPI_PUSHR_CTCNT_MASK | SPI_PUSHR_PCS(0x01) | SPI_PUSHR_EOQ_MASK;
			break;
	}
}
